#include <libxml/tree.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "partition.h"
#include "xml.h"

int add_partition(xmlElementPtr partition, partition_info *pi)
{
	if (!partition)
		return -1;

	get_val_str(partition, "Name", pi->szName, sizeof pi->szName);
	get_val_str(partition, "FileName", pi->szFileName, sizeof pi->szFileName);
	get_val_str(partition, "DeltaName", pi->szDeltaName, sizeof pi->szDeltaName);
	pi->nBinType = get_val_int(partition, "BinType");
	pi->nDevType = get_val_int(partition, "DevType");
	pi->nID = get_val_int(partition, "ID");
	pi->nAttribute = get_val_int(partition, "Attribute");
	pi->nUpdateAttr = get_val_int(partition, "UpdateAttr");
	pi->dwBlkSize = get_val_int(partition, "BlockSize");
	pi->dwBlkLen = get_val_int(partition, "BlockLength");
	pi->dwOffset = get_val_int(partition, "Offset");
	pi->dwFileSize = get_val_int(partition, "FileSize");

	return 0;
}

int xml2pit(const char *xmlfile, const char *pitfile)
{
	partition_header *hdr;
	xmlDocPtr doc; /* the resulting document tree */
	xmlNodePtr root_node, node;
	int partition_count = 0;
	int n = 0;
	unsigned char *pit;
	int sz;
	int r;
	int fd;

	doc = xmlReadFile(xmlfile, NULL, 0);
	if (!doc)
	{
		fprintf(stderr, "failed to parse %s\n", xmlfile);
		return -1;
	}

	root_node = xmlDocGetRootElement(doc);

	if (strcmp((const char*)(root_node->name), "root"))
	{
		fprintf(stderr, "invalid root element name\n");
		return -1;
	}

	/* count the partitions */
	for (node = root_node->children; node; node = node->next)
	{
		if (node->type == XML_ELEMENT_NODE &&
		    !strcmp((const char*)(node->name), "partition"))
		{
			partition_count++;
		}
	}

	sz = sizeof (partition_header) + sizeof (partition_info) * partition_count;
	pit = calloc(1, sz);
	if (!pit)
		return -1;

	/* fill the header */
	hdr = (void*) pit;
	hdr->dwMagic = PARTITION_MAGIC;
	hdr->nCount = partition_count;

	for (node = root_node->children; node; node = node->next)
	{
		if (node->type == XML_ELEMENT_NODE &&
		    !strcmp((const char*)(node->name), "partition"))
		{
			partition_info *pi;

			pi = (void*) &pit[sizeof (partition_header) + n * sizeof (partition_info)];
			r = add_partition((xmlElementPtr)node, pi);
			if (r)
			{
				return -1;
			}
			n++;
		}
	}

	xmlFreeDoc(doc);
	xmlCleanupParser();

	fd = creat(pitfile, 0666);
	if (fd < 0)
		return -1;

	r = write(fd, pit, sz);
	if (r != sz)
		return -1;

	close(fd);

	return 0;
}

int main(int argc, char **argv)
{

	if (argc != 3)
	{
		fprintf(stderr, "%s [xmlfile] [pitfilename]\n", argv[0]);
		return 1;
	}

	return xml2pit(argv[1], argv[2]) < 0 ? 1 : 0;
}
